require_relative 'bear_julius.rb'

# he needs a marriage and a women
class Married < ThingTank::Character
  property :date              # the date of the marriage
  property :end               # when the marriage ended
  property :spouse            # doc_id of the spouse
  property :state             # state of the marriage

  validates_presence_of :date
  validates_presence_of :spouse
  validate :spouse_should_be_a_person

  # ensure that 'spouse' is a doc_id of a Person
  def spouse_should_be_a_person
    Person.get(self["spouse"]).valid?
  end
end

# we want easy access to the name of the spouse
class Spouse < ThingTank::Character
  property :married
  property :married_state

  validate :spouse_should_be_married
  validates :married, :character => Married # doc must have "married" character
  
  def married
    self["married"]
  end

  def spouse_should_be_married
    married && married["spouse"] == self["_id"]
  end

  def name
    self["married_state"] == "married" ?
      Person.get(married["spouse"]).name :
      nil
  end

  def ex
    self["married_state"] == "divorced" ?
      Person.get(married["spouse"]).name :
      nil
  end
end

def test_first_marriage()
  julius = test_bear_julius
  conny = create :gender => "f", :name => 'Cornelia', :characters => ['Person']

  julius["married"] = {"date" => "84 BC", "spouse" => conny.id}
  julius["married_state"] = "married"

  assert julius.is(Married, "married").valid?
  assert julius.has(Spouse).valid?

  julius.save
  julius.reload
  
  assert_equal 'Cornelia', julius.has(Spouse).name

end


def test_first_marriage_shortcuts()
  julius = test_bear_julius
  conny = create :gender => "f", :name => 'Cornelia', :characters => ['Person']

  julius["married"] = {"date" => "84 BC", "spouse" => conny.id}
  julius["married_state"] = "married"

  assert (julius>Married-"married").valid?
  assert (julius>Spouse).valid?

  julius.save
  julius.reload
  
  assert_equal 'Cornelia', (julius>Spouse).name

end
