require_relative 'first_marriage.rb'

class Married

  # marry a doc or hash
  def marry(person)
    person = ThingTank.new(person) if person.is_a?(Hash)
    person.save

    # assign the doc_id to spouse
    self["spouse"] = person["_id"]
    self["state"] = 'married'
    _doc["married_state"] = 'married'
    
    unless person["married"] && person.last_character(Married, "married").spouse == _doc["_id"]
      person.add_character(Married, "married") do |m|
        m.date = self["date"]
        m.marry _doc
      end
      person.is(Spouse)
      person.save
    end
  end

  def divorce(date)
    self["state"] = 'divorced'
    self['end'] = date
    _doc.save
    spouse = Spouse.get(self["spouse"])
    if spouse.married_state == "married"
      spouse.divorce(date)
    end
  end

end

class Spouse
  def married(&code)
    _doc.last_character Married, "married", &code
  end

  def divorce(date)
    self["married_state"] = 'divorced'
    married { |m| m.divorce(date) }
  end
end

class Person
  def marry(date, person)
    _doc.add_character Married, "married"  do |m|
      m.date = "84 BC"
      m.marry person
    end
  end
end

def test_improved_marriage()
  julius = test_bear_julius()
  julius.as(Person).marry "84 BC", :gender => "f", :name => 'Cornelia'

  julius.save
  julius.reload
  
  assert_equal 'Cornelia',            julius.has(Spouse).name
  assert_equal 'married',             julius.has(Spouse).married_state

  conny = db_load julius.last_character(Married,"married").spouse

  assert_equal 'Gaius Iulius Caesar', conny.has(Spouse).name
  assert_equal 'married',             conny.has(Spouse).married_state

  assert_equal julius['_id'],         conny.last_character(Married,"married").spouse
  assert_equal "84 BC",               conny.last_character(Married,"married").date

  return julius
end

def test_improved_marriage_divorce()
  julius = test_bear_julius()
  julius.as(Person).marry "84 BC", :gender => "f", :name => 'Cornelia'
  julius.as(Spouse).divorce "85 BC"
  julius.save
  julius.reload

  assert_equal nil,                    julius.has(Spouse).name
  assert_equal 'Cornelia',             julius.has(Spouse).ex
  assert_equal 'divorced',             julius["married_state"]
  assert_equal "85 BC",                julius.last_character(Married,"married").end

  conny = db_load julius.last_character(Married,"married").spouse

  assert_equal nil,                    conny.has(Spouse).name
  assert_equal 'Gaius Iulius Caesar',  conny.has(Spouse).ex
  assert_equal 'divorced',             conny["married_state"]

  assert_equal julius['_id'],         conny.last_character(Married,"married").spouse
  assert_equal "85 BC",               conny.last_character(Married,"married").end
  return julius
end

def test_improved_marriage_shortcuts()
  julius = test_bear_julius()
  (julius>Person).marry "84 BC", :gender => "f", :name => 'Cornelia'

  julius.save
  julius.reload
  
  assert_equal 'Cornelia',            (julius>Spouse).name
  assert_equal 'married',             (julius>Spouse).married_state

  conny = db_load((julius>Married-"married").spouse)

  assert_equal 'Gaius Iulius Caesar', (conny>Spouse).name
  assert_equal 'married',             (conny>Spouse).married_state

  assert_equal julius['_id'],         (conny>Married-"married").spouse
  assert_equal "84 BC",               (conny>Married-"married").date

  return julius
end


