require_relative 'marriage_improvement.rb'

class Dead

  # all callbacks of characters are called and defined like corresponding callbacks of the doc
  before_save do
    if _doc.is?(Spouse) && _doc['married_state'] == 'married'
      Spouse.get(_doc.last_character(Married, 'married').spouse).widowed(self["date_of_death"])
    end
    true
  end

end

class Person
  def dies(date)
    _doc.is(Dead) do |d|
      d.date_of_death = date
    end
  end
end

class Married
  def widow(date)
    self["state"] = 'widowed'
    self['end'] = date
    _doc.save
  end
end

class Spouse
  def widowed(date)
    self["married_state"] = 'widowed'
    married { |m| m.widow(date) }
  end
end


def test_second_marriage()
  julius = test_improved_marriage()
  julius.as(Person).marry "68-65 BC", :gender => "f", :name => 'Pompeia'

  assert_equal 2, julius["married"].size
  assert_equal 'Cornelia', Person.get(julius["married"].first["spouse"]).name
  assert_equal 'Pompeia', Person.get(julius["married"].last["spouse"]).name
  assert_equal 'Pompeia', julius.has(Spouse).name
  assert_equal 'married', julius["married"].first["state"]
  assert_equal 'married', julius["married"].last["state"]

  return julius
end

def test_second_marriage_after_conny_died()
  julius = test_bear_julius()
  conny = create(:gender => "f", :name => 'Cornelia')
  conny.is(Person)
  julius.as(Person).marry "84 BC", conny
  julius.save
  julius.reload

  conny = db_load(julius["married"]["spouse"])
  conny.save
  conny.reload
  conny.as(Person).dies "68-65 BC"
  conny.save

  julius.reload
  assert_equal 'widowed', julius["married_state"]

  julius.as(Person).marry "68-65 BC", :gender => "f", :name => 'Pompeia'
  julius.save
  julius.reload
  assert_equal 2, julius["married"].size
  assert_equal 'Cornelia', Person.get(julius["married"].first["spouse"]).name
  assert_equal 'widowed', julius["married"].first["state"]
  assert_equal 'Pompeia', Person.get(julius["married"].last["spouse"]).name
  assert_equal 'married', julius["married"].last["state"]
  assert_equal 'Pompeia', julius.has(Spouse).name
end

