require 'test_helper'
require 'pp'

class Car < ThingTank::Character
  property :name
  property :project
end

class House < ThingTank::Character
  property :name
  property :project
end

$list_filter = "function(head, req) {
  var row;
  var rows = [];
  var debug = [];
  var list_filter = {};
  if(req.query['list_filter']){
    list_filter = JSON.parse(req.query['list_filter']);
  }
  while(row = getRow()) {    
    if(list_filter){
      var add = true;
      for(fi in list_filter){
        var filter_array = list_filter[fi];
        if(filter_array){
          var doc_val = row.doc;
          var filter_key = filter_array[0];
          var filter_val = filter_array[1];
          if(doc_val){
            var fkeys = filter_key.split('.'); /* filter_key may be data.page.disabled, so we have to get through that */
            for(fk in fkeys){
              if(doc_val !== undefined)
                doc_val = doc_val[fkeys[fk]];
            }
            if(doc_val !== filter_val){
              add = false;
            }  
          } else {
            if(Object.prototype.toString.call(row.value) === '[object Array]'){
              debug[debug.length] = ['array', row.value[fi], filter_val];
              if(row.value[fi] !== filter_val){
                add = false;
              }
              
            } else {
              debug[debug.length] = ['no_array', row.value, filter_val];
              if(row.value !== filter_val){
                add = false;
              }
            }
          }
        }
      }
      if(add)
        rows[rows.length] = row;      
    } else {
      rows[rows.length] = row;
    }
    
  };
  send(JSON.stringify({'rows': rows, 'debug': debug}));
}"


class Tanker < ThingTank
  design do
    list :testlist, $list_filter # only when called with .all() / include_docs=true
    #view :all_cars, :by => ['name']
    character_view House, :by_name, :by => ['name']
    character_view House, :by_special, :emit =>  "emit(doc.name, doc.project);"
    character_view Car, :by_name, :by => ['name']
    #character_view Car, :by_project, :emit => "emit(doc.project, 1);"
    character_view Car, :by_project_and_name, :by => ['project', 'name']

    character_view Car, :by_special, :map =>  "function(doc){ emit(doc.name, doc.project); }"
    character_view Car, :by_special2, :map =>  "function(doc){ emit(doc.name, [doc.project, doc.name]); }"
    character_view Car, :by_special3, :map =>  "function(doc){ emit(doc.name, [doc.name, doc.project]); }"
    character_view Car, :by_special4, :emit =>  "emit(doc.name, [doc.name, doc.project]);"
    character_view Car, :by_special5, :emit =>  "emit(doc.name, doc.project);"
  end
end


describe "with a car view" do

  before do
    reset_test_db!
  end

  it "should be able to handle rows without docs" do
    doc = Tanker.new {}
    doc.as(Car) do |c|
      c.project = 'a'
      c.name = 'A'
    end
    doc.save

    doc2 = Tanker.new {}
    doc2.as(House) do |c|
      c.project = 'b'
      c.name = 'B'
    end
    doc2.save

    doc3 = Tanker.new {}
    doc3.as(House) do |c|
      c.project = 'a'
      c.name = 'House a1'
    end
    doc3.save

    doc4 = Tanker.new {}
    doc4.as(Car) do |c|
      c.project = 'a'
      c.name = 'A1'
    end
    doc4.save

    doc5 = Tanker.new {}
    doc5.as(Car) do |c|
      c.project = 'a'
      c.name = 'A3'
    end
    doc5.save

    doc6 = Tanker.new {}
    doc6.as(House) do |c|
      c.project = 'b'
      c.name = 'B2'
    end
    doc6.save

    #assert_equal 3, Tanker.house_by_name.all().size
    assert_equal 3, Tanker.car_by_project_and_name.rows().size

    #assert_equal 4, Tanker.list_testlist('all', :list_filter => {'project' => 'a'}.to_json).all().size
    #assert_equal 2, Tanker.list_testlist('all', :list_filter => {'project' => 'b'}.to_json).all().size

    #result = Tanker.list_testlist('car_by_special', :list_filter => [['project', 'a']].to_json).execute()
    #if result['error']
    #  p [:error_project_a, result['reason'].split("\n").first]
    #else
    #  p [:debug_project_a, result['debug'], result['rows']]
    #end

    #p [:project_a, Tanker.list_testlist('car_by_special', :list_filter => [['project', 'a']].to_json).rows()]

    # should check the correct value if the value isn't an array but already our filter
    assert_equal 4, Tanker.list_testlist('car_by_special', :list_filter => [['project', 'a']].to_json).rows().size
    # same for docs
    assert_equal 4, Tanker.list_testlist('car_by_special', :list_filter => [['project', 'a']].to_json).all().size

    # should check the correct index in the values array, if the index is the first one
    assert_equal 4, Tanker.list_testlist('car_by_special2', :list_filter => [['project', 'a']].to_json).rows().size
    # same for docs
    assert_equal 4, Tanker.list_testlist('car_by_special2', :list_filter => [['project', 'a']].to_json).all().size

    # should check for the correct index in the values array, skipping nil filters
    assert_equal 4, Tanker.list_testlist('car_by_special3', :list_filter => [nil, ['project', 'a']].to_json).rows().size
    # same for docs
    assert_equal 4, Tanker.list_testlist('car_by_special3', :list_filter => [nil, ['project', 'a']].to_json).all().size


    # should check for the correct index in the values array, skipping nil filters
    assert_equal 3, Tanker.list_testlist('car_by_special4', :list_filter => [nil, ['project', 'a']].to_json).rows().size
    # same for docs
    assert_equal 3, Tanker.list_testlist('car_by_special4', :list_filter => [nil, ['project', 'a']].to_json).all().size

    assert_equal 1, Tanker.list_testlist('house_by_special', :list_filter => [['project', 'a']].to_json).all().size
    assert_equal 2, Tanker.list_testlist('house_by_special', :list_filter => [['project', 'b']].to_json).all().size
    assert_equal 3, Tanker.list_testlist('car_by_special5', :list_filter => [['project', 'a']].to_json).rows().size
    assert_equal 0, Tanker.list_testlist('car_by_special5', :list_filter => [['project', 'b']].to_json).all().size


  end

  it "should be able to use character_views as well as normal views" do
    doc = Tanker.new {}
    doc.as(Car) do |c|
      c.project = 'a'
      c.name = 'A'
    end
    doc.save

    doc2 = Tanker.new {}
    doc2.as(House) do |c|
      c.project = 'b'
      c.name = 'B'
    end
    doc2.save

    doc3 = Tanker.new {}
    doc3.as(House) do |c|
      c.project = 'a'
      c.name = 'House a1'
    end
    doc3.save

    doc4 = Tanker.new {}
    doc4.as(Car) do |c|
      c.project = 'a'
      c.name = 'A1'
    end
    doc4.save

    doc5 = Tanker.new {}
    doc5.as(Car) do |c|
      c.project = 'a'
      c.name = 'A3'
    end
    doc5.save

    doc6 = Tanker.new {}
    doc6.as(House) do |c|
      c.project = 'b'
      c.name = 'B2'
    end
    doc6.save

    assert_equal 3, Tanker.house_by_name.all().size
    assert_equal 3, Tanker.car_by_name.all().size

    assert_equal 4, Tanker.list_testlist('all', :list_filter => [['project', 'a']].to_json).all().size
    assert_equal 2, Tanker.list_testlist('all', :list_filter => [['project', 'b']].to_json).all().size

    #result = Tanker.list_testlist('house_by_name', :list_filter => {'project' => 'a'}.to_json, :include_docs => true).execute()
    #if result['error']
    #  p [:error, result['reason'].split("\n").first]
    #else
    #  p [:debug, result['debug']]
    #end

    assert_equal 1, Tanker.list_testlist('house_by_name', :list_filter => [['project', 'a']].to_json).all().size
    assert_equal 2, Tanker.list_testlist('house_by_name', :list_filter => [['project', 'b']].to_json).all().size
    assert_equal 3, Tanker.list_testlist('car_by_name', :list_filter => [['project', 'a']].to_json).all().size
    assert_equal 0, Tanker.list_testlist('car_by_name', :list_filter => [['project', 'b']].to_json).all().size

  end

  it "should do something with the list" do
    doc = Tanker.new {}
    doc.as(Car) do |c|
      c.project = 'a'
      c.name = 'A'
    end
    doc.save

    doc2 = Tanker.new {}
    doc2.as(Car) do |c|
      c.project = 'b'
      c.name = 'B'
    end
    doc2.save

    doc3 = Tanker.new {}
    doc3.as(Car) do |c|
      c.project = 'a'
      c.name = 'a2'
    end
    doc3.save

    
    #result = Tanker.list_testlist('all', :list_filter => {'project' => 'a'}.to_json, :include_docs => true).execute()
    #if result['error']
    #  p [:error, result['reason'].split("\n").first]
    #else
    #  p [:debug, result['debug']]
    #end
    assert_equal 3, Tanker.all.all().size
    assert_equal 2, Tanker.list_testlist('all', :list_filter => [['project', 'a']].to_json).all().size
    assert_equal 1, Tanker.list_testlist('all', :list_filter => [['project', 'b']].to_json).all().size
  end


  it "should have both without a filter list" do
    doc = Tanker.new {}
    doc.as(Car) do |c|
      c.project = 'My Project'
      c.name = 'My Name'
    end
    doc.save

    doc2 = Tanker.new {}
    doc2.as(Car) do |c|
      c.project = 'Other Project'
      c.name = 'My Name'
    end
    doc2.save
    assert_equal 2, Tanker.all.all().size
    assert_equal 2, Tanker.list_testlist('all').all().size
  end
end
