require 'test_helper'

class DrivingLicense < ThingTank::Character
  property :date_of_driving_license, String
  property :driving_license_state
  property :tester

  def driving_license_valid?
    self["tester"] = "huha"
    self["driving_license_state"] == "valid"
  end


  before_save do
    #p :before_save_licence
    true
  end
end

class Car < ThingTank::Character  
  property :color,     String
  property :model,     String

  wants DrivingLicense

  validates_presence_of :model
  validates_presence_of :color

  before_save do
    #p :before_save_car
    true
  end
end

class CarSub < Car
  property :additional
end

describe "a general test" do
  
  before do
    reset_test_db!    
  end

  it "shoud save under a special id" do
    doc = create :_id => ".config/projectname", :a => 'b'
    doc.save
    doc = db_load ".config/projectname"
    assert_equal ".config/projectname", doc.id
    assert_equal "b", doc["a"]
  end

  it "should inherit the properties" do
    doc = create
    doc.add_character CarSub do |c|
      c.color = 'green'
      c.additional = 'Spoiler'
    end
    doc.save
    doc.reload
    car_props = Car.character_properties.uniq
    car_props.delete "name"
    car_props.delete "project"
    car_sub_props = CarSub.character_properties.uniq
    car_sub_props.delete "name"
    car_sub_props.delete "project"
    assert_equal ["color", "model"], car_props.sort
    assert_equal ["additional", "color", "model"], car_sub_props.sort
    assert_equal false, doc.as(CarSub).valid?
    assert_equal 'green', doc.as(CarSub).color
  end

  it "should add a character if set by is but not if just asked if it could be a character" do
    doc1 = create
    doc1.is(DrivingLicense)
    could_it = doc1.could_be?(Car)
    doc1.save
    assert_equal false, could_it
    assert_equal ["DrivingLicense"], db_load(doc1.id)["characters"]
  end

  it "should work with with" do
    doc = create 
    doc.with('check') do |c|
      c.as(Check).a = 'A'
      c.as(Checker).checker1 = 'C'
    end

    assert_equal 'A', doc['check']['a']
    assert_equal 'C', doc['check']['checker1']
  end

  it "should work with 'with' and doc ids" do
    first = create :name => 'A thing'
    doc = create 'my_thing' => first.id
    doc.with 'my_thing' do |c|
      assert_equal 'A thing', c["name"]
      c["name"] = 'renamed'
    end

    first.reload
    assert_equal 'renamed', first["name"]
  end

  it "should work with with and subs" do
    doc = create 
    doc.with('check') do |c|
      c.with('sub') do |s|
        s.as(Check).a = 'subA'
        s.as(Checker).checker1 = 'subC'
      end
      c.as(Check).a = 'A'
      c.as(Checker).checker1 = 'C'
    end

    assert_equal 'A', doc['check']['a']
    assert_equal 'C', doc['check']['checker1']
    assert_equal 'subA', doc['check']['sub']['a']
    assert_equal 'subC', doc['check']['sub']['checker1']
  end

  it "should work with with and subs" do
    doc = create 'check' => []
    doc.with_first('check') do |c|
      c.with('sub1') do |s|
        s.as(Check).a = 'sub1A'
        s.as(Checker).checker1 = 'sub1C'
      end
      c.as(Check).a = 'A1'
      c.as(Checker).checker1 = 'C1'
    end

    doc["check"] << {}

    doc.with_last('check') do |c|
      c.with('sub2') do |s|
        s.as(Check).a = 'sub2A'
        s.as(Checker).checker1 = 'sub2C'
      end
      c.with('sub3') do |s|
        s.as(Check).a = 'sub3A'
        s.as(Checker).checker1 = 'sub3C'
      end
      c.as(Check).a = 'A2'
      c.as(Checker).checker1 = 'C2'
    end

    doc.with_first('check') do |c|
      c.as(Check).a = 'A-changed'
    end

    assert_equal 'A-changed', doc['check'].first['a']
    assert_equal 'C1', doc['check'].first['checker1']
    assert_equal 'sub1A', doc['check'].first['sub1']['a']
    assert_equal 'sub1C', doc['check'].first['sub1']['checker1']

    assert_equal 'A2', doc['check'].last['a']
    assert_equal 'C2', doc['check'].last['checker1']
    assert_equal 'sub2A', doc['check'].last['sub2']['a']
    assert_equal 'sub2C', doc['check'].last['sub2']['checker1']

    assert_equal 'sub3A', doc['check'].last['sub3']['a']
    assert_equal 'sub3C', doc['check'].last['sub3']['checker1']
  end


  it "make the changed test" do
    doc = create :a => "A", :characters => ["Check"]
    assert doc.is? Check
    assert !doc.as(Check).changed?
    chk = doc.as(Check) do |c|
      c.a = 'AAA'
    end

    assert_equal false, chk.changed?
    assert_equal 'AAA', doc["a"]

    doc["a"] = 'AA'

    assert_equal 'AA', doc["a"]
    assert_equal 'AA', chk.reload["a"]
    assert_equal 'AA', doc.as(Check)["a"]
    doc.save
    doc.reload
    assert_equal 'AA', doc["a"]
  end

  it "should export a property to its doc" do
    other = { "sub" => "hash" }
    doc = create :weather => "fine", :other => other
    assert_equal other, doc["other"]
    doc.export "other"
    doc2 = db_load doc["other_id"]
    assert_equal "fine", doc["weather"]
    assert_equal "hash", doc2["sub"]
    assert_equal nil, doc["other"]
  end

  it "should allow same access if property is a thing or if it is a array of things or if it is a Hash" do
    doc = create "thingie" => "hiho"
    assert_equal "hiho", doc["thingie"]
    doc["thingie"] = ["huhu", "hiho"]
    doc.save
    doc.reload
    assert_equal ["huhu", "hiho"], doc["thingie"]
    doc["thingie"] = { "hi" => "ho" }
    doc.save
    doc.reload
    assert_equal({ "hi" => "ho" }, doc["thingie"])
  end

  it "is a car" do
    doc = create :color => 'black', :model => 'Carrera', :characters => ['Car']
    assert_equal true, doc.is?(Car)
  end

  it "cannot be a car" do
    doc = create :model => 'Carrera'
    doc["characters"] = ['Car']
    assert_equal false, doc.is?(Car)
    assert_equal ["Car"], doc.invalid_characters
    #assert_raises RuntimeError do
    #  doc.save
    #end
    doc["color"] = 'blue'
    assert_equal true, doc.is?(Car)
    assert_equal [], doc.invalid_characters
    doc.save
    doc.reload
    assert_equal true, doc.is?(Car)

    doc = create :color => 'black', :model => 'Carrera'
    assert_equal false, doc.is?(Car)
  end

  it "could be a car" do
    doc = create :color => 'black', :model => 'Carrera'
    assert_equal true, doc.could_be?(Car)
  end

  it "should wish a driving license" do
    assert_equal [DrivingLicense], Car.wishes
  end

  it "should have a driving license" do
    doc = create :date_of_driving_license => Time.now.to_s, :driving_license_state => 'valid'
    assert !doc.has?(DrivingLicense)
    doc.have(DrivingLicense) do |licence|
      assert_equal true, licence.driving_license_valid?
    end
    assert doc.has?(DrivingLicense)
    doc.save
    assert_equal "huha", doc["tester"]
    assert_equal [DrivingLicense], Car.wishes
  end


  #it "should try to validate something" do
  #  doc = CouchRest::Document.new :hi => "ho"
  #  p doc.valid?
  #end

end

=begin
describe "a thingtank" do

  before do
    #skip "temporary disabled"
    reset_test_db!
  end

  after do
  end
  
  it "should allow same access if property is a thing or if it is a array of things" do
  end

  it "should allow same access if property is a thing or if it is the doc_id of a thing" do
  end

  it "should allow same access if property is a array of things or if it is array of doc_ids that are things" do
  end

  it "should be able to change the name of a property" do
  end

  it "should be able to share properties with different characters" do
  end

  it "should trigger all affected callbacks when a property is shared between characters" do
  end

  it "should allow an alias for a property if the property needs disambiguation between the different meanings for the different characters" do
  end

  it "should allow general handlers for characters that only are affected if a doc fits to a character and ignore the docs otherwise" do
  end

  it "should allow views that handle only the chosen characters" do
  end

  it "should recommend interesting new characters" do
  end

  it "should validate with validatable" do
  end

  it "should convert basic types as Time" do
  end

  it "should handle pagination" do
  end

  it "should have characters" do
  end

end



=end
