require 'test_helper'
require 'pp'

class Car < ThingTank::Character
  property :name
end

class House < ThingTank::Character
  property :name
end

class Ship < ThingTank::Character
  property :name
  property :weight
end


class Tanker < ThingTank

  design do
    character_view Car, :by_name, :by => ['name']
    character_view House, :by_name, :by => ['name']
    character_view Ship, :by_name, :by => ['name']
    character_view Ship, :by_name_with_weight, :emit => 'emit(doc.name, doc.weight);'
    character_view Ship, :by_name_with_weight2, :emit => 'emit(doc.name, [doc.name, doc.weight]);'
  end
end


describe "with a car view" do

  before do
    reset_test_db!    
  end

  it "should get ships with name and weight as rows" do
    ship = Tanker.create
    ship.as(Ship) do |s|
      s.name = "MS Dolphin"
      s.weight = "1000 pound"
    end    
    ship.save

    result = Tanker.ship_by_name_with_weight.rows()

    assert_equal 1, result.size
    assert_equal 'MS Dolphin', result.first.key
    assert_equal '1000 pound', result.first.value

    result = Tanker.ship_by_name_with_weight2.rows()

    assert_equal 1, result.size
    assert_equal 'MS Dolphin', result.first.key
    assert_equal 'MS Dolphin', result.first.value.first
    assert_equal '1000 pound', result.first.value.last

  end

  it "should get all cars" do
    car1 = Tanker.create 
    car1.as(Car) do |c|
      c.name = "Volvo"
    end

    car2 = Tanker.create 
    car2.as(Car) do |c|
      c.name = "Ferrari"
    end

    ship = Tanker.create
    ship.as(Ship) do |s|
      s.name = "MS Dolphin"
      s.weight = "1000 pound"
    end
    ship.is(House) # too!

    ship.save

    house = Tanker.create
    house.as(House) do |h|
      h.name = "Villa Kunterbunt"
    end

    house.save

    car1.save
    car2.save

    cars = Tanker.car_by_name.rows
    houses = Tanker.house_by_name.rows
    ships = Tanker.ship_by_name.rows

    assert_equal 2, cars.size
    assert_equal "Ferrari", cars.first["key"]
    assert_equal "Volvo", cars.last["key"]
    assert_equal 2, houses.size # MS Dolphin is a House too!
    assert_equal "Villa Kunterbunt", houses.last["key"]
    assert_equal "MS Dolphin", houses.first["key"]
    assert_equal 1, ships.size
    assert_equal "MS Dolphin", ships.first["key"]
  end

end

